# See full reference at https://devenv.sh/reference/options/
{
  pkgs,
  inputs,
  phps,
  lib,
  config,
  modulesPath,
  ...
}: let
  
  
  taskfileYaml = let
       
    ciJobToken = builtins.getEnv "CI_JOB_TOKEN";
    gitCommit=  if ciJobToken == "" then
      (builtins.getFlake "git+https://gitlab.schukai.com/schukai/entwicklung/nix-flakes").packages.${builtins.currentSystem}.git-commit
    else
     ""; 
       
    goPkgReleaseData = import ./project.nix;
    goPkgName = goPkgReleaseData.name or "goPkg";
    goPkgMnemonic = goPkgReleaseData.mnemonic or "go-pkg";       
  in
  pkgs.writeTextFile {
    name = "Taskfile.yaml";
    text = ''
## DO NOT EDIT THIS FILE MANUALLY, IT IS GENERATED BY NIX
version: '3'

tasks:
    build:
      desc: Build the app
      aliases:
        - b
      cmds:
        - build-app
      sources:
        - source/*.go
        - source/**/*.go
        - source/go.mod
        - dist/**
        
    git-commit:
        desc: Commit changes to git
        cmds:
          - ${gitCommit}/bin/git-commit
        silent: true
    
    default:
      desc: Print this help message
      aliases: [d, help]
      cmds:
        - task --list
      silent: true

## DO NOT EDIT THIS FILE MANUALLY, IT IS GENERATED BY NIX
    '';
  };

  treefmtConfig = pkgs.writeTextFile {
    name = "treefmt.toml";
    text = ''
        ## DO NOT EDIT THIS FILE MANUALLY, IT IS GENERATED BY NIX

        [formatter.nix]
        command = "${pkgs.alejandra}/bin/alejandra"
        options = []
        includes = [ "*.nix" ]
        excludes = [ "devenv.nix" ]

        [formatter.go]
        command = "${pkgs.go}/bin/gofmt"
        options = ["-w"]
        includes = [ "*.go" ]
        excludes = []

        #[formatter.php]
        #command = "${pkgs.php83Packages.php-cs-fixer}/bin/php-cs-fixer"
        #options = ["fix"]
        #includes = [ "*.php" ]
        #excludes = []

        [formatter.shell]
        command = "${pkgs.shfmt}/bin/shfmt"
        options = ["-w"]
        includes = [ "*.sh", "*.bash" ]
        excludes = []


        [formatter.prettier]
        command = "${pkgs.nodePackages.prettier}/bin/prettier"
        options = ["--write","--plugin=${pkgs.nodePackages.prettier-plugin-toml}/lib/node_modules/prettier-plugin-toml/lib/api.js"]
        includes = [
            "*.css",
            "*.html",
            "*.js",
            "*.mjs",
            "*.json",
            "*.xml",
            "*.toml",
            "*.jsx",
            "*.md",
            "*.mdx",
            "*.scss",
            "*.ts",
            "*.yml",
            "*.yaml"]
        excludes = ["treefmt.toml", "Taskfile.yml", "node_modules"]

      ## DO NOT EDIT THIS FILE MANUALLY, IT IS GENERATED BY NIX

    '';
  };

  gitlabCiYaml = pkgs.writeTextFile {
    name = ".gitlab-ci.yml";
    text = ''
        ## DO NOT EDIT THIS FILE MANUALLY, IT IS GENERATED BY NIX
        image: docker-registry.schukai.com:443/nixos-ci-devenv:latest

        services:
          - docker:dind


        variables:
          NIXOS_VERSION: "23.11"
          NIXPKGS_ALLOW_UNFREE: "1"
          NIXPKGS_ALLOW_INSECURE: "1"
          DOCKER_DRIVER: overlay2


        stages:
          - tag
          - release


        tag-it:
          tags:
            - nixos
          stage: tag
          script:
            - devenv shell tag-version

          rules:
            - if: '$CI_COMMIT_TAG'
              when: never
            - if: '$CI_COMMIT_MESSAGE =~ /RELEASE_IT/'
              when: never
            - when: on_success


        release:
          tags:
            - nixos
          stage: release
          rules:
            - if: '$CI_COMMIT_MESSAGE =~ /RELEASE_IT/'
              when: on_success
            - when: never
          variables:
            # Disable shallow cloning so that goreleaser  can diff between tags to
            GIT_DEPTH: 0
          script:
            - devenv shell deploy-app
            #- cd source; goreleaser release --clean
      ## DO NOT EDIT THIS FILE MANUALLY, IT IS GENERATED BY NIX
    '';
  };

  goReleaserYaml = let 
    goPkgReleaseData = import ./project.nix;
    goPkgName = goPkgReleaseData.name or "goPkg";
    goPkgMnemonic = goPkgReleaseData.mnemonic or "go-pkg";
  in
  pkgs.writeTextFile {
    name = ".goreleaser.yml";
    text = ''
      ## DO NOT EDIT THIS FILE MANUALLY, IT IS GENERATED BY NIX
      gitlab_urls:
        api: https://gitlab.schukai.com/api/v4
        download: https://gitlab.schukai.com

        # set to true if you use a self-signed certificate
        skip_tls_verify: false

        use_package_registry: true

        # Set this if you set GITLAB_TOKEN to the value of CI_JOB_TOKEN.
        # Since: v1.11
        use_job_token: false

      project_name: ${goPkgMnemonic}

      builds:
        - id: ${goPkgMnemonic}
          goos:
            - linux
            - darwin
            - windows
          dir: ${config.devenv.root}/source
          ldflags:
            - -s -w 
            - -X main.version={{.Version}}
            - -X 'main.commit={{.Commit}}' 
            - -X 'main.build={{.Date}}' 
            - -X 'main.mnemonic=${goPkgMnemonic}' 
            - -X 'main.name=${goPkgName}'


      ## DO NOT EDIT THIS FILE MANUALLY, IT IS GENERATED BY NIX
    '';
  };
in {
  env.APP_NAME =  let 
                     goPkgReleaseData = import ./project.nix;
                     goPkgMnemonic = goPkgReleaseData.mnemonic or "go-pkg";
                   in goPkgMnemonic;

  # https://devenv.sh/packages/
  packages = with pkgs; [
    inputs.version.defaultPackage."${builtins.currentSystem}"
    appimage-run
    goreleaser
    go-tools
    awscli2
    blackbox
    blackbox-terminal
    coreutils-full
    d2
    dbeaver
    delve
    dialog
    dive
    docker-client
    docker-compose
    drill
    eza
    fd
    gawk
    gcc12
    gdlv
    git
    glab
    gnugrep
    gnum4
    gnumake
    gnupg
    gnused
    go-task
    graphviz
    treefmt
    gum
    httpie
    hurl
    jq
    libffi
    libffi
    logrotate
    mdbook
    mdbook-admonish
    mdbook-cmdrun
    mdbook-d2
    mdbook-emojicodes
    mdbook-graphviz
    mdbook-linkcheck
    mdbook-mermaid
    mdbook-pdf
    meld
    memcached
    netcat
    netcat
    oha
    openai
    oxker
    pandoc-katex
    plantuml
    pprof
    procps
    ranger
    termbook
    unixtools.arp
    unixtools.xxd
    unzip
    util-linux
    wget
    zlib
  ];

  scripts.tag-version.exec = ''
    #!${pkgs.bash}/bin/bash

    if [[ -z "''${CI_JOB_TOKEN}" ]]; then
        echo " ✖ Not in gitlab ci."
        exit 0
    fi
    
    set -x
    ${pkgs.git}/bin/git config user.email "''${GITLAB_USER_EMAIL}"
    ${pkgs.git}/bin/git config user.name "''${GITLAB_USER_NAME:-"Gitlab CI"}"
    ${pkgs.git}/bin/git config credential.helper '!f() { echo "username=gitlab-ci-token"; echo "password=''${CI_JOB_TOKEN}"; }; f'
    ${pkgs.git}/bin/git config pull.rebase true
    ${pkgs.git}/bin/git config http.sslVerify "false"
    ${pkgs.git}/bin/git remote set-url origin https://pad:''${GITLAB_TOKEN}@''${CI_REPOSITORY_URL#*@}
    ${pkgs.git}/bin/git fetch --all --tags --unshallow
    ${pkgs.git}/bin/git reset --hard origin/master
    ${pkgs.git}/bin/git checkout $CI_COMMIT_REF_NAME
    ${pkgs.git}/bin/git pull origin $CI_COMMIT_REF_NAME
    
    NEWVERSION=$(${inputs.version.defaultPackage."${builtins.currentSystem}"}/bin/version predict --git exit-code-if-no-bump)
    if [[ $? -ne 0 ]]; then
    echo " ✖ No bump wanted."
    exit 0
    fi
    
    if [[ -z "''${NEWVERSION}" ]]; then
    echo " ✖ No version found."
    exit 0
    fi
    
    sed -i "s/version\s*=\s*\".*\"/version=\"''${NEWVERSION}\"/" ${config.devenv.root}/project.nix
    ${pkgs.git}/bin/git add ${config.devenv.root}/project.nix
    ${pkgs.git}/bin/git commit -m "chore: Bump version to ''${NEWVERSION}"
    
    ${pkgs.git}/bin/git log --decorate=short --pretty=oneline
    
    ${pkgs.git}/bin/git tag -a "''${NEWVERSION}" -m "chore: Release ''${NEWVERSION} - RELEASE_IT"
    ${pkgs.git}/bin/git push origin $CI_COMMIT_REF_NAME --tags

  '';
  
  scripts.common-functions.exec = ''

    if [ -t 1 ]; then
        RED='\033[0;31m'
        GREEN='\033[0;32m'
        BLUE='\033[0;34m'
        RESET='\033[0m'
        BOLD='\033[1m'
    else
        RED=""
        GREEN=""
        RESET=""
    fi

    is_true() {
      val="$1"
      if [ "$val" = 'true' ] || [ "$val" = '1' ]; then
        return 0
      else
        return 1
      fi
    }

    get_commit_hash() {
        hash=""
        if [[ -n "$CI_COMMIT_SHORT_SHA" ]]; then
            hash="$CI_COMMIT_SHORT_SHA"
        else
            hash="$(git rev-parse --short HEAD)"

            if [[ -n "$(git status --porcelain)" ]]; then
                hash="''${hash}-dirty"
            fi
        fi
        echo "$hash"
    }
    
    echo_green() {
        echo -e "$GREEN ✔ $1$RESET"
    }
    
    echo_red() {
        echo -e "$RED ✖ $1$RESET"
    }
    
    echo_todo() {
        echo_bold "$BLUE • $1"
    }

    update_project_version() {
        if version auto --git --exit-code-if-no-bump --verbose
        then
            buildVer=$(version print --git)
            local project_nix="${config.devenv.root}/project.nix"
            sed -i "s/^\s*version\s*=.*/  version = \"$buildVer\";/" "$project_nix"
            echo_green "Project version updated to $buildVer"

            ${pkgs.git}/bin/git add "$project_nix"
            ${pkgs.git}/bin/git commit -m "chore: Update project version to $buildVer" "$project_nix"
            ${pkgs.git}/bin/git tag -f -a "$buildVer" -m "Release $buildVer"
            
            if is_true "$CI_MODE"; then
                ${pkgs.git}/bin/git push -o ci.skip origin ''${CI_COMMIT_REF_NAME} --tags
            fi
            
        else
            echo_green "Project version is up to date"
        fi
    }

    get_project_version() {
        echo "$(version print --git)"
    }

    if [ -z "$DEBUG_MODE" ]; then
        DEBUG_MODE='false'
    fi

    if [ -n "$CI_JOB_TOKEN" ]; then
        CI_MODE='true'
    fi

    if is_true "$DEBUG_MODE" || is_true "$CI_MODE"; then
        set -x
    fi

  '';
  
  
  scripts.update-files.exec = ''
    #!${pkgs.bash}/bin/bash
    source common-functions
    create_symlink() {
        if ln -s "$1" "$2" 2>/dev/null; then
            echo_green "$2 created"
        else
            echo_red "$2 already exists"
        fi
    }
    
    update_symlink() {
        echo -e " → $1"
        local source_path="$1"
        local target_path="$2"
        local file_name="$(basename "$target_path")"
        local source_dir="$(dirname "$source_path")"
        mkdir -p "$source_dir" || true
        if [[ ! -d "$source_dir" ]]; then
         echo_red "$RED ✖ $source_dir is not a directory. Please remove it manually and run the script again."
         exit 1
        fi
        
        if [[ -L "$source_path" ]]; then
        if [[ "$(readlink "$source_path")" != "$target_path" ]]; then
             rm "$source_path"
             create_symlink "$target_path" "$source_path"
             echo_green "$GREEN ✔ $source_path updated$RESET"
         else
                echo_green "$GREEN ✔ $source_path is up to date$RESET"
         fi
        elif [[ -e "$source_path" ]]; then
         echo_red "$RED ✖ $source_path already exists and is not a symlink. Please remove it manually and run the script again."
         exit 1
        else
         create_symlink "$target_path" "$source_path"
        fi
    }

    # Usage for Taskfile.yaml
    update_symlink "${config.devenv.root}/Taskfile.yaml" "${taskfileYaml}"
    update_symlink "${config.devenv.root}/.goreleaser.yml" "${goReleaserYaml}"
    update_symlink "${config.devenv.root}/treefmt.toml" "${treefmtConfig}"

    cat ${gitlabCiYaml} > "${config.devenv.root}/.gitlab-ci.yml"

  '';

  languages = {
    go = {enable = true;};
  };

  difftastic.enable = true;

  scripts.get-commit-hash.exec = ''
    #!${pkgs.bash}/bin/bash
        hash=""
        if [[ -n "$CI_COMMIT_SHORT_SHA" ]]; then
            hash="$CI_COMMIT_SHORT_SHA"
        else
            hash="$(git rev-parse --short HEAD)"

            if [[ -n "$(git status --porcelain)" ]]; then
                hash="''${hash}-dirty"
            fi
        fi
        echo "$hash"
  '';

  scripts.build-app.exec = ''
    #!${pkgs.bash}/bin/bash
    ${pkgs.treefmt}/bin/treefmt
    update-hashes
    ${pkgs.nix}/bin/nix build .#version
  '';

  scripts.deploy-app.exec = ''
    #!${pkgs.bash}/bin/bash
    ${pkgs.goreleaser}/bin/goreleaser release --clean --config ${config.devenv.root}/.goreleaser.yml
  '';

  scripts.update-hashes.exec = ''
    #!${pkgs.bash}/bin/bash
    sed -i 's/vendorHash\s*=\s*".*"/vendorHash="sha256-AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA="/' ${config.devenv.root}/project.nix
    HASH=$(${pkgs.nix}/bin/nix build 2>&1 | grep 'got:'  | awk '{print $2}')
    sed -i "s/vendorHash\s*=\s*\".*\"/vendorHash=\"''${HASH}\"/" ${config.devenv.root}/project.nix
  '';

  enterShell = ''
    update-files
  '';
}
